try:
  import psyco
  psyco.profile()
except:
  pass
    
import os 
from math import *

def test1(protdir, prot):
  classfile = protdir + os.sep + ".." + os.sep + "classification.list"
  rec = [l.split() for l in open(classfile).readlines() if l.split()[0] == prot][0]
  return "Complex difficulty: %s" % rec[1]
  
def test2(protdir, prot):
  classfile = protdir + os.sep + ".." + os.sep + "classification.list"
  rec = [l.split() for l in open(classfile).readlines() if l.split()[0] == prot][0] 
  return "Complex class: %s" % rec[2]

def interface(protdir, prot, chain):
  cdfile = protdir + os.sep + prot+chain+".contactdis"  
  lines = [l.split() for l in open(cdfile).readlines() if len(l.split()) == 2]
  return [int(l[0]) for l in lines if float(l[1]) < 5]

def ifsize(protdir, prot, chain):
  return len(interface(protdir, prot, chain))

def surface(protdir, prot, chain):
  surfile = protdir + os.sep + "predictions" + os.sep + prot+chain+".sur"  
  return [int(l) for l in open(surfile).readlines() if len(l.split())]
  
def sursize(protdir, prot, chain):
  return len(surface(protdir, prot, chain))

def chaintest1(protdir, prot, chain):
  return "Interface size: %d" % ifsize(protdir, prot, chain)

def chaintest2(protdir, prot, chain):
  return "Number of surface residues: %d" % sursize(protdir, prot, chain)

def chaintest3(protdir, prot, chain):
  i = ifsize(protdir, prot, chain)
  s = sursize(protdir, prot, chain)
  return "Interface fraction: %.3f" % (float(i)/s)

def preds(protdir, prot, chain, predfile):
  preds = [l.split() for l in open(predfile).readlines() if len(l.split())]
  if len(preds[0]) == 2:
    sur = surface(protdir, prot, chain)
    preds = [(float(l[0]), int(l[1])) for l in preds]
    preds = [(list(l[0])+[l[1]]) for l in zip(preds, sur)]
    preds = [l for l in preds if l[0] != -999]
    preds.sort(key=lambda x: x[0])
    preds = [l[2] for l in preds[-30:]]
  else:
    preds = [int(l[0]) for l in preds]
  return preds

def predtest1(protdir, prot, chain, predfile):
  p = preds(protdir, prot, chain, predfile)
  if len(p) == 0: return
  return "Number of predictions: %d" % len(p)

def predtest2(protdir, prot, chain, predfile):
  iface = interface(protdir, prot, chain)
  p = preds(protdir, prot, chain, predfile)
  if len(p) == 0: return
  return "Sensitivity: %.3f" % (float(len(set(p).intersection(set(iface))))/ float(len(iface)))

def predtest3(protdir, prot, chain, predfile):
  iface = interface(protdir, prot, chain)
  p = preds(protdir, prot, chain, predfile)
  if len(p) == 0: return
  return "Specificity: %.3f" % (float(len(set(p).intersection(set(iface))))/ float(len(p)))

def predtest4(protdir, prot, chain, predfile):
  iface = set(interface(protdir, prot, chain))
  p = set(preds(protdir, prot, chain, predfile))
  if len(p) == 0: return
  sur = surface(protdir, prot, chain)
  tp = len(p.intersection(iface))
  fp = len(p) - tp
  fn = len(iface) - tp
  tn = len(sur) - tp - fp - fn
  mcc_num = tp*tn-fp*fn
  mcc_denom = sqrt((tp+fp)*(tp+fn)*(tn+fp)*(tn+fn))
  if mcc_denom == 0:
    if mcc_num == 0: mcc = 0
    else: mcc = mcc_num / abs(mcc_num)
  else: mcc = mcc_num / mcc_denom
  return "Matthews correlation coeffient: %.3f" % (mcc)


from stattools import hypergeom
  
def predtest5(protdir, prot, chain, predfile):
  iface = set(interface(protdir, prot, chain))
  p = set(preds(protdir, prot, chain, predfile))
  if len(p) == 0: return
  sur = surface(protdir, prot, chain)
  tp = len(p.intersection(iface))
  fp = len(p) - tp
  fn = len(iface) - tp
  tn = len(sur) - tp - fp - fn  
  pval = 0
  #print tp+fn, len(sur), tp, tp+fp
  for n in range(0,min(fn+1,fp+1)):
    pval += hypergeom(tp+n, fp-n, tn+n, fn-n)
  return "P-value of prediction being better than random (hypergeom dist): %.3g" % (pval)

def predtest6(protdir, prot, chain, predfile):
  iface = set(interface(protdir, prot, chain))
  p = set(preds(protdir, prot, chain, predfile))
  if len(p) == 0: return
  sur = surface(protdir, prot, chain)
  tp = len(p.intersection(iface))
  fp = len(p) - tp
  fn = len(iface) - tp  
  tn = len(sur) - tp - fp - fn
  pval = 0
  for n in range(0,min(tp+1,tn+1)):
    pval += hypergeom(tp-n, fp+n, tn-n, fn+n)
  return "P-value of prediction being worse than random (hypergeom dist): %.3g" % (pval)
